/*
 * Copyright 2018 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.stunner.bpmn.project.backend.workitem.deploy;

import java.util.Collections;

import org.guvnor.common.services.project.model.Dependencies;
import org.guvnor.common.services.project.model.POM;
import org.guvnor.common.services.project.service.POMService;
import org.guvnor.common.services.shared.metadata.MetadataService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.kie.workbench.common.services.shared.project.KieModule;
import org.kie.workbench.common.services.shared.project.KieModuleService;
import org.kie.workbench.common.stunner.bpmn.workitem.WorkItemDefinition;
import org.kie.workbench.common.stunner.core.diagram.Metadata;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.uberfire.backend.vfs.Path;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class WorkItemDefinitionProjectInstallerTest {

    private static final WorkItemDefinition WID = new WorkItemDefinition()
            .setName("testWID")
            .setDependencies(new Dependencies(Collections.emptyList()));

    @Mock
    private POMService pomService;

    @Mock
    private MetadataService metadataService;

    @Mock
    private KieModuleService moduleService;

    @Mock
    private Metadata metadata;

    @Mock
    private Path root;

    private WorkItemDefinitionProjectInstaller tested;

    @Before
    public void init() {
        when(metadata.getRoot()).thenReturn(root);
        tested = new WorkItemDefinitionProjectInstaller(pomService,
                                                        metadataService,
                                                        moduleService);
    }

    @Test
    public void testInstall() {
        KieModule module = mock(KieModule.class);
        Path pomXMLPath = mock(Path.class);
        POM pom = mock(POM.class);
        when(moduleService.resolveModule(eq(root))).thenReturn(module);
        when(module.getPomXMLPath()).thenReturn(pomXMLPath);
        when(pomService.load(eq(pomXMLPath))).thenReturn(pom);
        Dependencies dependencies = new Dependencies(Collections.emptyList());
        when(pom.getDependencies()).thenReturn(dependencies);
        org.guvnor.common.services.shared.metadata.model.Metadata projMetadata =
                mock(org.guvnor.common.services.shared.metadata.model.Metadata.class);
        when(metadataService.getMetadata(pomXMLPath)).thenReturn(projMetadata);
        tested.install(Collections.singleton(WID),
                       this.metadata);
        verify(pomService, times(1))
                .save(eq(pomXMLPath),
                      eq(pom),
                      eq(projMetadata),
                      anyString(),
                      eq(false));
    }
}
